Stan Math Library: stan/math/prim/scal/prob/beta_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

beta_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_BETA_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_BETA_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/log1m.hpp>
13 #include <stan/math/prim/scal/fun/multiply_log.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/fun/digamma.hpp>
16 #include <stan/math/prim/scal/fun/lgamma.hpp>
17 #include <stan/math/prim/scal/fun/lbeta.hpp>
18 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
19 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
20 #include <stan/math/prim/scal/fun/constants.hpp>
21 #include <stan/math/prim/scal/meta/include_summand.hpp>
22 #include <stan/math/prim/scal/fun/grad_reg_inc_beta.hpp>
23 #include <stan/math/prim/scal/fun/inc_beta.hpp>
24 #include <boost/math/special_functions/gamma.hpp>
25 #include <boost/random/gamma_distribution.hpp>
26 #include <boost/random/variate_generator.hpp>
27 #include <cmath>
28
29 namespace stan {
30
31 namespace math {
32
51 template <bool propto,
52 typename T_y, typename T_scale_succ, typename T_scale_fail>
53 typename return_type<T_y, T_scale_succ, T_scale_fail>::type
54 beta_log(const T_y& y,
55 const T_scale_succ& alpha, const T_scale_fail& beta) {
56 static const char* function("stan::math::beta_log");
57
58 typedef typename stan::partials_return_type<T_y,
59 T_scale_succ,
60 T_scale_fail>::type
61 T_partials_return;
62
63 using stan::math::digamma;
64 using stan::math::lgamma;
65
66 using stan::is_constant_struct;
67 using stan::is_vector;
68 using stan::math::check_positive_finite;
69 using stan::math::check_not_nan;
70 using stan::math::check_consistent_sizes;
71 using stan::math::include_summand;
72 using stan::math::log1m;
73 using stan::math::multiply_log;
74 using stan::math::value_of;
75 using stan::math::check_nonnegative;
76 using stan::math::check_less_or_equal;
77 using std::log;
78
79 // check if any vectors are zero length
80 if (!(stan::length(y)
81 && stan::length(alpha)
82 && stan::length(beta)))
83 return 0.0;
84
85 // set up return value accumulator
86 T_partials_return logp(0.0);
87
88 // validate args (here done over var, which should be OK)
89 check_positive_finite(function, "First shape parameter", alpha);
90 check_positive_finite(function, "Second shape parameter", beta);
91 check_not_nan(function, "Random variable", y);
92 check_consistent_sizes(function,
93 "Random variable", y,
94 "First shape parameter", alpha,
95 "Second shape parameter", beta);
96 check_nonnegative(function, "Random variable", y);
97 check_less_or_equal(function, "Random variable", y, 1);
98
99 // check if no variables are involved and prop-to
100 if (!include_summand<propto, T_y, T_scale_succ, T_scale_fail>::value)
101 return 0.0;
102
103 VectorView<const T_y> y_vec(y);
104 VectorView<const T_scale_succ> alpha_vec(alpha);
105 VectorView<const T_scale_fail> beta_vec(beta);
106 size_t N = max_size(y, alpha, beta);
107
108 for (size_t n = 0; n < N; n++) {
109 const T_partials_return y_dbl = value_of(y_vec[n]);
110 if (y_dbl < 0 || y_dbl > 1)
111 return LOG_ZERO;
112 }
113
114 // set up template expressions wrapping scalars into vector views
115 OperandsAndPartials<T_y, T_scale_succ, T_scale_fail>
116 operands_and_partials(y, alpha, beta);
117
118 VectorBuilder<include_summand<propto, T_y, T_scale_succ>::value,
119 T_partials_return, T_y>
120 log_y(length(y));
121 VectorBuilder<include_summand<propto, T_y, T_scale_fail>::value,
122 T_partials_return, T_y>
123 log1m_y(length(y));
124
125 for (size_t n = 0; n < length(y); n++) {
126 if (include_summand<propto, T_y, T_scale_succ>::value)
127 log_y[n] = log(value_of(y_vec[n]));
128 if (include_summand<propto, T_y, T_scale_fail>::value)
129 log1m_y[n] = log1m(value_of(y_vec[n]));
130 }
131
132 VectorBuilder<include_summand<propto, T_scale_succ>::value,
133 T_partials_return, T_scale_succ>
134 lgamma_alpha(length(alpha));
135 VectorBuilder<!is_constant_struct<T_scale_succ>::value,
136 T_partials_return, T_scale_succ>
137 digamma_alpha(length(alpha));
138 for (size_t n = 0; n < length(alpha); n++) {
139 if (include_summand<propto, T_scale_succ>::value)
140 lgamma_alpha[n] = lgamma(value_of(alpha_vec[n]));
141 if (!is_constant_struct<T_scale_succ>::value)
142 digamma_alpha[n] = digamma(value_of(alpha_vec[n]));
143 }
144
145 VectorBuilder<include_summand<propto, T_scale_fail>::value,
146 T_partials_return, T_scale_fail>
147 lgamma_beta(length(beta));
148 VectorBuilder<!is_constant_struct<T_scale_fail>::value,
149 T_partials_return, T_scale_fail>
150 digamma_beta(length(beta));
151
152 for (size_t n = 0; n < length(beta); n++) {
153 if (include_summand<propto, T_scale_fail>::value)
154 lgamma_beta[n] = lgamma(value_of(beta_vec[n]));
155 if (!is_constant_struct<T_scale_fail>::value)
156 digamma_beta[n] = digamma(value_of(beta_vec[n]));
157 }
158
159 VectorBuilder<include_summand<propto, T_scale_succ, T_scale_fail>::value,
160 T_partials_return, T_scale_succ, T_scale_fail>
161 lgamma_alpha_beta(max_size(alpha, beta));
162
163 VectorBuilder<contains_nonconstant_struct<T_scale_succ,
164 T_scale_fail>::value,
165 T_partials_return, T_scale_succ, T_scale_fail>
166 digamma_alpha_beta(max_size(alpha, beta));
167
168 for (size_t n = 0; n < max_size(alpha, beta); n++) {
169 const T_partials_return alpha_beta = value_of(alpha_vec[n])
170 + value_of(beta_vec[n]);
171 if (include_summand<propto, T_scale_succ, T_scale_fail>::value)
172 lgamma_alpha_beta[n] = lgamma(alpha_beta);
173 if (contains_nonconstant_struct<T_scale_succ, T_scale_fail>::value)
174 digamma_alpha_beta[n] = digamma(alpha_beta);
175 }
176
177 for (size_t n = 0; n < N; n++) {
178 // pull out values of arguments
179 const T_partials_return y_dbl = value_of(y_vec[n]);
180 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
181 const T_partials_return beta_dbl = value_of(beta_vec[n]);
182
183 // log probability
184 if (include_summand<propto, T_scale_succ, T_scale_fail>::value)
185 logp += lgamma_alpha_beta[n];
186 if (include_summand<propto, T_scale_succ>::value)
187 logp -= lgamma_alpha[n];
188 if (include_summand<propto, T_scale_fail>::value)
189 logp -= lgamma_beta[n];
190 if (include_summand<propto, T_y, T_scale_succ>::value)
191 logp += (alpha_dbl-1.0) * log_y[n];
192 if (include_summand<propto, T_y, T_scale_fail>::value)
193 logp += (beta_dbl-1.0) * log1m_y[n];
194
195 // gradients
196 if (!is_constant_struct<T_y>::value)
197 operands_and_partials.d_x1[n] += (alpha_dbl-1)/y_dbl
198 + (beta_dbl-1)/(y_dbl-1);
199 if (!is_constant_struct<T_scale_succ>::value)
200 operands_and_partials.d_x2[n]
201 += log_y[n] + digamma_alpha_beta[n] - digamma_alpha[n];
202 if (!is_constant_struct<T_scale_fail>::value)
203 operands_and_partials.d_x3[n]
204 += log1m_y[n] + digamma_alpha_beta[n] - digamma_beta[n];
205 }
206 return operands_and_partials.value(logp);
207 }
208
209 template <typename T_y, typename T_scale_succ, typename T_scale_fail>
210 inline typename return_type<T_y, T_scale_succ, T_scale_fail>::type
211 beta_log(const T_y& y, const T_scale_succ& alpha,
212 const T_scale_fail& beta) {
213 return beta_log<false>(y, alpha, beta);
214 }
215
216 }
217 }
218 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::beta_log
return_type< T_y, T_scale_succ, T_scale_fail >::type beta_log(const T_y &y, const T_scale_succ &alpha, const T_scale_fail &beta)
The log of the beta density for the specified scalar(s) given the specified sample size(s)...
Definition: beta_log.hpp:54
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::is_vector
Definition: is_vector.hpp:10
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::partials_return_type
Definition: partials_return_type.hpp:16
check_less_or_equal.hpp
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_less_or_equal
bool check_less_or_equal(const char *function, const char *name, const T_y &y, const T_high &high)
Return true if y is less or equal to high.
Definition: check_less_or_equal.hpp:81
stan::contains_nonconstant_struct
Definition: contains_nonconstant_struct.hpp:14
grad_reg_inc_beta.hpp
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
lbeta.hpp
inc_beta.hpp
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::log1m
fvar< T > log1m(const fvar< T > &x)
Definition: log1m.hpp:16
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.